@@ -9,6 +9,7 @@ module.exports = function ( data, files ){
 	uvs = [],
 	faces = [],
 	materials = [],
+	colors = [],
 	material = "",
 
 	// numbers
@@ -171,7 +172,16 @@ module.exports = function ( data, files ){
 
 	// post processing
 	// - include material information (create loop for more than one files)
-	materials = material_parser( files[0], materials );
+	materials = material_parser( files[0], materials, this.options );
+	// count colors
+	if( this.options.bake_colors ){
+		colors = extract_material_colors( JSON.parse(materials) );
+		// convert to hexcolor
+		for( c in colors){
+			colors[c] = generate_color_decimal( colors[c] );
+		}
+		colorCount = colors.length;
+	}
 
 	// final result
 	return {
@@ -188,6 +198,7 @@ module.exports = function ( data, files ){
 		vertices : vertices,
 		uvs : uvs,
 		normals : normals,
+		colors : colors,
 		materials : materials,
 		mtllib : mtllib
 	}
@@ -247,32 +258,62 @@ function parse_vertex( line ){
 
 }
 
+// Extract diffuse colors from MTL materials
+function extract_material_colors( materials ){
+	// fallbacks
+	materials = materials || [{ 'default': 0 }];
+	var colors = [];
+	for( m in materials){
+		var material = materials[m];
+		//var index
+		var color = material["colorDiffuse"] || [1,0,0];
+		if( material["DbgIndex"] ){
+			var index = material["DbgIndex"];
+			colors[index]= color;
+		} else {
+			colors.push(color);
+		}
+	}
+	colors.sort();
+
+	return colors;
+}
+
+function generate_color_decimal(c){
+	//return "%d" % hexcolor(c);
+	return hexcolor(c);
+}
+
+function hexcolor(c){
+	return ( parseInt(c[0] * 255, 10) << 16  ) + ( parseInt(c[1] * 255, 10) << 8 ) + parseInt(c[2] * 255, 10);
+}
+
 // Regular expressions
 
 // v float float float
 
-var vertex_pattern = /v( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)/;
+var vertex_pattern = /v( +[\d|\.|\+|\-|e|E]+)( +[\d|\.|\+|\-|e|E]+)( +[\d|\.|\+|\-|e|E]+)/;
 
 // vn float float float
 
-var normal_pattern = /vn( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)/;
+var normal_pattern = /vn( +[\d|\.|\+|\-|e|E]+)( +[\d|\.|\+|\-|e|E]+)( +[\d|\.|\+|\-|e|E]+)/;
 
 // vt float float
 
-var uv_pattern = /vt( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)/
+var uv_pattern = /vt( +[\d|\.|\+|\-|e|E]+)( +[\d|\.|\+|\-|e|E]+)/;
 
 // f vertex vertex vertex ...
 
-var face_pattern1 = /f( +\d+)( +\d+)( +\d+)( +\d+)?/
+var face_pattern1 = /f( +-?\d+)( +-?\d+)( +-?\d+)( +-?\d+)?/;
 
 // f vertex/uv vertex/uv vertex/uv ...
 
-var face_pattern2 = /f( +(\d+)\/(\d+))( +(\d+)\/(\d+))( +(\d+)\/(\d+))( +(\d+)\/(\d+))?/;
+var face_pattern2 = /f( +(-?\d+)\/(-?\d+))( +(-?\d+)\/(-?\d+))( +(-?\d+)\/(-?\d+))( +(-?\d+)\/(-?\d+))?/;
 
 // f vertex/uv/normal vertex/uv/normal vertex/uv/normal ...
 
-var face_pattern3 = /f( +(\d+)\/(\d+)\/(\d+))( +(\d+)\/(\d+)\/(\d+))( +(\d+)\/(\d+)\/(\d+))( +(\d+)\/(\d+)\/(\d+))?/;
+var face_pattern3 = /f( +(-?\d+)\/(-?\d+)\/(-?\d+))( +(-?\d+)\/(-?\d+)\/(-?\d+))( +(-?\d+)\/(-?\d+)\/(-?\d+))( +(-?\d+)\/(-?\d+)\/(-?\d+))?/;
 
 // f vertex//normal vertex//normal vertex//normal ...
 
-var face_pattern4 = /f( +(\d+)\/\/(\d+))( +(\d+)\/\/(\d+))( +(\d+)\/\/(\d+))( +(\d+)\/\/(\d+))?/;
+var face_pattern4 = /f( +(-?\d+)\/\/(-?\d+))( +(-?\d+)\/\/(-?\d+))( +(-?\d+)\/\/(-?\d+))( +(-?\d+)\/\/(-?\d+))?/;
